/** 
    This utility class contains, after initialization, references to
    naming service, logger, storage and control unit
*/

import Backup.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

public class Global {
    
  protected static NamingContextExt nm;
  protected static boolean initialized = false;
  protected static Logger logger;

  /** 
        Count of miliseconds between retries
  */
  protected static int retry_milis = 1000; // 1 sec.
  
  /** 
        Count of retries before final failure
  */
  protected static int retry_count = 10 ;   // -1 - unlimited retries

  public synchronized void RetryCount (int rc) {
    retry_count=rc;
  }
  
  public synchronized int RetryCount () {
    return retry_count;
  }
  
  public synchronized void RetryTime (int miliseconds) {
    retry_milis=miliseconds;
  }
  
  public synchronized int RetryTime () {
    return retry_milis;
  }
 
  /** 
      Decrements retry count and sleeps for retry_milis ms;
      returns true, if another retry is possible.
  */
  protected synchronized static boolean Retry(int retries) {
      if (retries > 0 || retries == -1) {
        System.err.print("Remaining retries: " + retries +' ');
        try {
          Thread.sleep(retry_milis);
        } catch (InterruptedException e) {}
        System.err.println("retrying... ");
        return true; 
      } else {
        System.err.println("Giving up retries");
        return false;
      }
  }
  
  /**
      Checks if the naming is initialized
  */
  public synchronized static boolean CheckNaming() {
    if (nm == null) {
      return false;
    } else {
      return true;
    }
  }
  
  /** 
      Initialize connection to naming service
  */
  
  public synchronized static boolean Initialize(ORB orb) {
    return Initialize(orb, retry_count);
  }
  
  protected synchronized static boolean Initialize(ORB orb, int retries) {
    try {
      org.omg.CORBA.Object obj;
      obj = orb.resolve_initial_references("NameService");
      if (obj == null) {
        System.err.println("Null reference to NameService");
        return false;
      }
      nm = NamingContextExtHelper.narrow(obj);
    } catch (Exception e) {
      System.err.println("Unable to obtain initial reference to NameService "
                         + e
      );
      if ( retries == -1 || Retry(retries--) ) {
        return Initialize(orb, retries); 
      } else {
        return false;
      }
    }
    return true;
  }

  /** 
      Gets the naming service
  */
  public synchronized static NamingContextExt GetNaming() 
                                  throws NotInitializedException {
    if (CheckNaming()) {
      return nm;
    } else {
      throw new NotInitializedException("Naming service not initialized");
    }
  }
  
  /** 
       Logs message via Logger 
  */
  public static void LogMessage(String caller, String message) {
  //  int retry=retry_count;
  //  do {
      try {
        if (logger == null) {
          if (! CheckNaming()) {
            System.err.println("Naming not initialized, logging to err");
            System.err.println( caller.toString() + ": " + message);
          }
          NameComponent nc [] = { new NameComponent("Logger"," ") }; 
          logger=LoggerHelper.narrow(nm.resolve(nc));
        }
        logger.logMessage(caller, message);
        return;
      } catch (Exception e) {
        System.err.println("Error while trying to log message: " + e); 
        logger = null; // Pro pripad chyby
      }
//    } while (Retry(retry--));
    System.err.println( caller.toString() + ": " + message);
  }
  
  /** 
       Gets reference to refname
  */
  public static org.omg.CORBA.Object Get(String refname) 
      throws NotInitializedException {
    int retry=retry_count;
    do {
      try {
        if (! CheckNaming()) {
          System.err.println("Naming not initialized, aborting Get"+refname);
          throw new NotInitializedException
             ("Naming service not initialized, Get: "+refname);
        }
        NameComponent nc [] = { new NameComponent(refname," ") }; 
        return nm.resolve(nc);
      } catch (Exception e) {
        System.err.println("Error while trying to get"+refname+": " + e); 
        return null; // Pro pripad chyby
      }
    } while (Retry(retry--));
  }
  
  public static Control GetControl() throws NotInitializedException {
     return ControlHelper.narrow(Get("Control"));
  }
  
  public static Storage GetStorage() throws NotInitializedException {
     return StorageHelper.narrow(Get("Storage"));
  }

  public static Agent GetAgent(String agentname) 
                        throws NotInitializedException {
     return AgentHelper.narrow(Get("Agent_"+ agentname));
  }
  
  public static boolean bind(String name, org.omg.CORBA.Object obj) 
                               throws AlreadyBound, NotInitializedException {
     if (! CheckNaming()) {
       System.err.println("Naming not initialized, aborting Register: "
                           +name);
       throw new NotInitializedException
          ("Naming service not initialized, Register: " + name);
     } else {
       NameComponent nc [] = { new NameComponent(name," ") }; 
       try {
         nm.bind(nc, obj);
       } catch (Exception e) {
         if ( e instanceof AlreadyBound) {
           throw (AlreadyBound) e;
         } else {
           return false;
         }
       }
     }
     return true;
  }

}
